@@ -1,16 +1,18 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id$ */
+
 /* Default implementation of device get_bits[_rectangle] */
 #include "memory_.h"
 #include "gx.h"
@@ -130,15 +132,7 @@ gx_get_bits_return_pointer(gx_device * dev, int x, int h,
          * the device wants something else, it should implement
          * get_bits_rectangle itself.
          */
-        uint dev_raster =
-            (both & GB_PACKING_CHUNKY ?
-               gx_device_raster(dev, true) :
-             both & GB_PACKING_PLANAR ?
-               bitmap_raster(dev->color_info.depth /
-                             dev->color_info.num_components * dev->width) :
-             both & GB_PACKING_BIT_PLANAR ?
-               bitmap_raster(dev->width) :
-             0 /* not possible */);
+        uint dev_raster = gx_device_raster(dev, true);
         uint raster =
             (options & (GB_RASTER_STANDARD | GB_RASTER_ANY) ? dev_raster :
              params->raster);
