@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2009 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Interface to platform-specific routines */
 /* Requires gsmemory.h */
 
@@ -19,6 +21,7 @@
 #  define gp_INCLUDED
 
 #include "gstypes.h"
+#include "gscdefs.h"		/* for gs_serialnumber */
 /*
  * This file defines the interface to ***ALL*** platform-specific routines,
  * with the exception of the thread/synchronization interface (gpsync.h)
@@ -95,6 +98,14 @@ const char *gp_strerror(int);
  */
 int gp_defaultpapersize(char *ptr, int *plen);
 
+/*
+ * Return a serialnumber. Clients that want to can modify the appropriate
+ * gp_***.c file(s) for their platform and Digital Rights Management (DRM)
+ * of choice. Default handlers for common platforms use info from the OS
+ * and unsupported or old platforms simply return GS_SERIALNUMBER.
+ */
+int gp_serialnumber(void);
+
 /* ------ Date and time ------ */
 
 /*
@@ -166,8 +177,13 @@ const char *gp_getenv_display(void);
  * parameter, but it would break too many clients to make this change now.)
  * Note that this is the size of the buffer, not the maximum number of
  * characters: the latter is one less, because of the terminating \0.
+ *
+ * This used to be 260, the same as the MAX_PATH value on Windows,
+ * but although MAX_PATH still exists on Windows, it is no longer
+ * the maximum length of a path - doh??
+ * We now use 4k as a reasonable limit for most environments.
  */
-#define gp_file_name_sizeof 260 /* == MAX_PATH on Windows */
+#define gp_file_name_sizeof 4096
 
 /* Define the character used for separating file names in a list. */
 extern const char gp_file_name_list_separator;
@@ -295,7 +311,7 @@ const char *gp_file_name_parent(void);
 /* Answer whether the platform allows parent refenences. */
 /*	unix, Win, Mac: yes */
 /*	VMS:	no.         */
-bool gp_file_name_is_partent_allowed(void);
+bool gp_file_name_is_parent_allowed(void);
 
 /* Answer whether an empty item is meanful in file names on the platform. */
 /*	unix, Win:  no	    */
@@ -310,6 +326,9 @@ bool gp_file_name_is_empty_item_meanful(void);
 int gp_read_macresource(byte *buf, const char *fname,
                                      const uint type, const ushort id);
 
+/* Returns true when the character can be used in the file name. */
+bool gp_file_name_good_char(unsigned char c);
+
 /* ------ persistent cache interface ------ */
 
 /*
@@ -340,8 +359,6 @@ int gp_cache_query(int type, byte* key, int keylen, void **buffer,
 /* cache data types */
 #define GP_CACHE_TYPE_TEST 0
 #define GP_CACHE_TYPE_FONTMAP 1
-#define GP_CACHE_TYPE_WTS_SIZE 2
-#define GP_CACHE_TYPE_WTS_CELL 3
 
 /* ------ Printer accessing ------ */
 
@@ -477,6 +494,8 @@ int64_t gp_ftell_64(FILE *strm);
 
 int gp_fseek_64(FILE *strm, int64_t offset, int origin);
 
+bool gp_fseekable (FILE *f);
+
 /* We don't define gp_fread_64, gp_fwrite_64,
    because (1) known platforms allow regular fread, fwrite
    to be applied to a file opened with O_LARGEFILE,
@@ -485,4 +504,11 @@ int gp_fseek_64(FILE *strm, int64_t offset, int origin);
    in one operation.
  */
 
+/* Some platforms (currently only windows) may supply a function to convert
+ * characters from an encoded command line arg from the local encoding into
+ * a unicode codepoint. Returns EOF for end of file (or string).
+ */
+int
+gp_local_arg_encoding_get_codepoint(FILE *file, const char **astr);
+
 #endif /* gp_INCLUDED */
