@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* CIE color rendering dictionary creation */
 #include "math_.h"
 #include "memory_.h"
@@ -383,7 +385,7 @@ typedef struct encode_data_s {
 
 /* Define procedures that retrieve the Encode values read from the list. */
 static float
-encode_from_data(floatp v, const float values[gx_cie_cache_size],
+encode_from_data(double v, const float values[gx_cie_cache_size],
                  const gs_range * range)
 {
     return (v <= range->rmin ? values[0] :
@@ -396,7 +398,7 @@ encode_from_data(floatp v, const float values[gx_cie_cache_size],
  * my craw, but I've got a mandate not to use macros....
  */
 static float
-encode_lmn_0_from_data(floatp v, const gs_cie_render * pcrd)
+encode_lmn_0_from_data(double v, const gs_cie_render * pcrd)
 {
     const encode_data_t *data = pcrd->client_data;
 
@@ -404,7 +406,7 @@ encode_lmn_0_from_data(floatp v, const gs_cie_render * pcrd)
                             &pcrd->DomainLMN.ranges[0]);
 }
 static float
-encode_lmn_1_from_data(floatp v, const gs_cie_render * pcrd)
+encode_lmn_1_from_data(double v, const gs_cie_render * pcrd)
 {
     const encode_data_t *data = pcrd->client_data;
 
@@ -412,7 +414,7 @@ encode_lmn_1_from_data(floatp v, const gs_cie_render * pcrd)
                             &pcrd->DomainLMN.ranges[1]);
 }
 static float
-encode_lmn_2_from_data(floatp v, const gs_cie_render * pcrd)
+encode_lmn_2_from_data(double v, const gs_cie_render * pcrd)
 {
     const encode_data_t *data = pcrd->client_data;
 
@@ -420,7 +422,7 @@ encode_lmn_2_from_data(floatp v, const gs_cie_render * pcrd)
                             &pcrd->DomainLMN.ranges[2]);
 }
 static float
-encode_abc_0_from_data(floatp v, const gs_cie_render * pcrd)
+encode_abc_0_from_data(double v, const gs_cie_render * pcrd)
 {
     const encode_data_t *data = pcrd->client_data;
 
@@ -428,7 +430,7 @@ encode_abc_0_from_data(floatp v, const gs_cie_render * pcrd)
                             &pcrd->DomainABC.ranges[0]);
 }
 static float
-encode_abc_1_from_data(floatp v, const gs_cie_render * pcrd)
+encode_abc_1_from_data(double v, const gs_cie_render * pcrd)
 {
     const encode_data_t *data = pcrd->client_data;
 
@@ -436,7 +438,7 @@ encode_abc_1_from_data(floatp v, const gs_cie_render * pcrd)
                             &pcrd->DomainABC.ranges[1]);
 }
 static float
-encode_abc_2_from_data(floatp v, const gs_cie_render * pcrd)
+encode_abc_2_from_data(double v, const gs_cie_render * pcrd)
 {
     const encode_data_t *data = pcrd->client_data;
 
