@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* CMap character decoding */
 #include "memory_.h"
 #include "string_.h"
@@ -335,6 +337,13 @@ gs_cmap_alloc(gs_cmap_t **ppcmap, const gs_memory_struct_type_t *pstype,
     return 0;
 }
 
+int gs_cmap_free(gs_cmap_t *pcmap, gs_memory_t *mem)
+{
+    gs_free_object(mem, pcmap->CIDSystemInfo, "gs_cmap_free(CIDSystemInfo)");
+    gs_free_object(mem, pcmap, "gs_cmap_free(CMap)");
+    return 0;
+}
+
 /*
  * Initialize an enumerator with convenient defaults (index = 0).
  */
@@ -558,7 +567,7 @@ gs_cmap_ToUnicode_alloc(gs_memory_t *mem, int id, int num_codes, int key_size, g
         char sid[10], *pref = "aux-";
         int sid_len, pref_len = strlen(pref);
 
-        sprintf(sid, "%d", id);
+        gs_sprintf(sid, "%d", id);
         sid_len = strlen(sid);
         name_len = pref_len + sid_len;
         cmap_name = gs_alloc_string(mem, name_len, "gs_cmap_ToUnicode_alloc");
@@ -573,8 +582,10 @@ gs_cmap_ToUnicode_alloc(gs_memory_t *mem, int id, int num_codes, int key_size, g
         return code;
     map = (uchar *)gs_alloc_bytes(mem, num_codes * gs_cmap_ToUnicode_code_bytes,
                                   "gs_cmap_ToUnicode_alloc");
-    if (map == NULL)
+    if (map == NULL) {
+        gs_cmap_free(*ppcmap, mem);
         return_error(gs_error_VMerror);
+    }
     memset(map, 0, num_codes * gs_cmap_ToUnicode_code_bytes);
     cmap = (gs_cmap_ToUnicode_t *)*ppcmap;
     cmap->glyph_name_data = map;
@@ -587,6 +598,13 @@ gs_cmap_ToUnicode_alloc(gs_memory_t *mem, int id, int num_codes, int key_size, g
     return 0;
 }
 
+int gs_cmap_ToUnicode_free(gs_memory_t *mem, gs_cmap_t *pcmap)
+{
+    gs_free_object(mem, pcmap->glyph_name_data, "Free ToUnicode glyph data");
+    gs_cmap_free(pcmap, mem);
+    return 0;
+}
+
 /*
  * Write a code pair to ToUnicode CMap.
  */
