@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Implementation of FunctionType 4 (PostScript Calculator) Functions */
 #include "math_.h"
 #include "memory_.h"
@@ -60,7 +62,7 @@ typedef struct calc_value_s {
 
 /* Store a float. */
 static inline void
-store_float(calc_value_t *vsp, floatp f)
+store_float(calc_value_t *vsp, double f)
 {
     vsp->value.f = f;
     vsp->type = CVT_FLOAT;
@@ -240,12 +242,12 @@ fn_PtCr_evaluate(const gs_function_t *pfn_common, const float *in, float *out)
             /* Coerce and re-dispatch */
 
         case PtCr_int_to_float:
-            store_float(vsp, (floatp)vsp->value.i);
+            store_float(vsp, (double)vsp->value.i);
             --p; goto sw;
         case PtCr_int2_to_float:
-            store_float(vsp, (floatp)vsp->value.i);
+            store_float(vsp, (double)vsp->value.i);
         case PtCr_2nd_int_to_float:
-            store_float(vsp - 1, (floatp)vsp[-1].value.i);
+            store_float(vsp - 1, (double)vsp[-1].value.i);
             --p; goto sw;
 
             /* Arithmetic operators */
@@ -349,7 +351,7 @@ fn_PtCr_evaluate(const gs_function_t *pfn_common, const float *in, float *out)
         case PtCr_neg_int:
         neg_int:
             if (vsp->value.i == min_int)
-                store_float(vsp, (floatp)vsp->value.i); /* =self negated */
+                store_float(vsp, (double)vsp->value.i); /* =self negated */
             else
                 vsp->value.i = -vsp->value.i;
             continue;
@@ -680,15 +682,15 @@ calc_access(const gs_data_source_t *psrc, ulong start, uint length,
     stream_SFD_state st;
     stream ds, bs;
     byte dbuf[200];		/* arbitrary */
-    const stream_template *const template = &s_SFD_template;
+    const stream_template *const templat = &s_SFD_template;
 
     /* Set up the stream that writes into the buffer. */
     s_init(&bs, NULL);
     swrite_string(&bs, buf, length);
     /* Set up the SubFileDecode stream. */
     s_init(&ds, NULL);
-    s_init_state((stream_state *)&st, template, NULL);
-    template->set_defaults((stream_state *)&st);
+    s_init_state((stream_state *)&st, templat, NULL);
+    templat->set_defaults((stream_state *)&st);
     st.skip_count = start;
     s_init_filter(&ds, (stream_state *)&st, dbuf, sizeof(dbuf), &bs);
     calc_put(&ds, pfn);
