@@ -236,6 +236,10 @@ gs_type42_font_init(gs_font_type42 * pfont, int subfontID)
 
             READ_SFNTS(pfont, offset, 30, maxp);
             pfont->data.trueNumGlyphs = U16(maxp + 4);
+            pfont->data.maxPoints = U16(maxp + 6);
+            pfont->data.maxContours = U16(maxp + 8);
+            pfont->data.maxCPoints = U16(maxp + 12);
+            pfont->data.maxCContours = U16(maxp + 16);
         } else if (!memcmp(tab, "name", 4)) {
             pfont->data.name_offset = offset;
         } else if (!memcmp(tab, "vhea", 4)) {
@@ -1345,7 +1349,7 @@ append_simple(const byte *gdata, float sbw[4], const gs_matrix_fixed *pmat,
             gs_fixed_point start,pt_start_off;
             gs_fixed_point cpoints[2];
 
-            if_debug1('1', "[1t]start %d\n", i);
+            if_debug1m('1', pfont->memory, "[1t]start %d\n", i);
 
             for (; np <= last_point; --reps, ++np) {
                 gs_fixed_point dpt;
@@ -1387,8 +1391,9 @@ append_simple(const byte *gdata, float sbw[4], const gs_matrix_fixed *pmat,
 
                 if (ppath) {
                     /* append to a path */
-                    if_debug3('1', "[1t]%s (%g %g)\n",
-                        (flags & gf_OnCurve ? "on " : "off"), fixed2float(pt.x), fixed2float(pt.y));
+                    if_debug3m('1', pfont->memory, "[1t]%s (%g %g)\n",
+                               (flags & gf_OnCurve ? "on " : "off"),
+                               fixed2float(pt.x), fixed2float(pt.y));
 
                     if (move) {
                         if(is_start_off) {
