@@ -24,6 +24,8 @@
 #include "gxclpath.h"
 #include "gxdevsop.h"
 
+extern dev_proc_dev_spec_op(gdev_prn_forwarding_dev_spec_op);
+
 /* ---------------- Writing utilities ---------------- */
 
 #define cmd_set_rect(rect)\
@@ -50,7 +52,7 @@ cmd_put_rect(register const gx_cmd_rect * prect, register byte * dp)
 
 int
 cmd_write_rect_hl_cmd(gx_device_clist_writer * cldev, gx_clist_state * pcls,
-                      int op, int x, int y, int width, int height, 
+                      int op, int x, int y, int width, int height,
                       bool extended_command)
 {
     byte *dp;
@@ -58,7 +60,7 @@ cmd_write_rect_hl_cmd(gx_device_clist_writer * cldev, gx_clist_state * pcls,
     int rcsize;
 
     cmd_set_rect(pcls->rect);
-    if (extended_command) { 
+    if (extended_command) {
         rcsize = 2 + cmd_size_rect(&pcls->rect);
         code = set_cmd_put_op(dp, cldev, pcls, cmd_opv_extend, rcsize);
         dp[1] = op;
@@ -70,8 +72,9 @@ cmd_write_rect_hl_cmd(gx_device_clist_writer * cldev, gx_clist_state * pcls,
     }
     if (code < 0)
         return code;
-    if_debug5('L', "rect hl r%d:%d,%d,%d,%d\n", 
-        rcsize - 1, pcls->rect.x, pcls->rect.y, pcls->rect.width, pcls->rect.height);
+    if_debug5m('L', cldev->memory, "rect hl r%d:%d,%d,%d,%d\n",
+               rcsize - 1, pcls->rect.x, pcls->rect.y,
+               pcls->rect.width, pcls->rect.height);
     cmd_put_rect(&pcls->rect, dp);
     return 0;
 }
@@ -124,14 +127,14 @@ cmd_write_rect_cmd(gx_device_clist_writer * cldev, gx_clist_state * pcls,
             code = set_cmd_put_op(dp, cldev, pcls, op + 0x10, 3);
             if (code < 0)
                 return code;
-            if_debug3('L', "    rs2:%d,%d,0,%d\n",
-                      dx, dwidth, dheight);
+            if_debug3m('L', cldev->memory, "    rs2:%d,%d,0,%d\n",
+                       dx, dwidth, dheight);
         } else {
             code = set_cmd_put_op(dp, cldev, pcls, op + 0x10, 5);
             if (code < 0)
                 return code;
-            if_debug4('L', "    rs4:%d,%d,%d,%d\n",
-                      dx, dwidth, dy, dheight);
+            if_debug4m('L', cldev->memory, "    rs4:%d,%d,%d,%d\n",
+                       dx, dwidth, dy, dheight);
             dp[3] = dy - rmin;
             dp[4] = dheight - rmin;
         }
@@ -157,8 +160,8 @@ cmd_write_rect_cmd(gx_device_clist_writer * cldev, gx_clist_state * pcls,
         code = set_cmd_put_op(dp, cldev, pcls, op, rcsize);
         if (code < 0)
             return code;
-        if_debug5('L', "    r%d:%d,%d,%d,%d\n",
-                  rcsize - 1, dx, dwidth, dy, dheight);
+        if_debug5m('L', cldev->memory, "    r%d:%d,%d,%d,%d\n",
+                   rcsize - 1, dx, dwidth, dy, dheight);
         cmd_put_rect(&pcls->rect, dp + 1);
     }
     return 0;
@@ -178,7 +181,7 @@ cmd_write_page_rect_cmd(gx_device_clist_writer * cldev, int op)
     gx_clist_state *pcls1;
     int code;
 
-    if_debug0('L', "[L]fillpage beg\n");
+    if_debug0m('L', cldev->memory, "[L]fillpage beg\n");
     code = set_cmd_put_all_op(dp, cldev, op, rcsize);
     if (code < 0)
         return code;
@@ -189,7 +192,7 @@ cmd_write_page_rect_cmd(gx_device_clist_writer * cldev, int op)
     cmd_putw(0, dp);
     cmd_putw(0, dp);
     cmd_putw(0, dp);
-    if_debug0('L', "[L]fillpage end\n");
+    if_debug0m('L', cldev->memory, "[L]fillpage end\n");
     return 0;
 }
 
@@ -274,10 +277,10 @@ cmd_write_trapezoid_cmd(gx_device_clist_writer * cldev, gx_clist_state * pcls,
         cmd_putw(ybot, dp);
         cmd_putw(ytop, dp);
     }
-    if_debug6('L', "    t%d:%ld,%ld,%ld,%ld   %ld\n",
-              rcsize - 1, left->start.x, left->start.y, left->end.x, left->end.y, ybot);
-    if_debug6('L', "    t%ld,%ld,%ld,%ld   %ld   o=%d\n",
-              right->start.x, right->start.y, right->end.x, right->end.y, ytop, options);
+    if_debug6m('L', cldev->memory, "    t%d:%ld,%ld,%ld,%ld   %ld\n",
+               rcsize - 1, left->start.x, left->start.y, left->end.x, left->end.y, ybot);
+    if_debug6m('L', cldev->memory, "    t%ld,%ld,%ld,%ld   %ld   o=%d\n",
+               right->start.x, right->start.y, right->end.x, right->end.y, ytop, options);
     if (options & 2) {
         cmd_putw(fa->clip->p.x, dp);
         cmd_putw(fa->clip->p.y, dp);
@@ -301,10 +304,18 @@ cmd_write_trapezoid_cmd(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 int
 clist_fillpage(gx_device * dev, gs_imager_state *pis, gx_drawing_color *pdcolor)
 {
-    gx_device_clist_writer * const cdev = &((gx_device_clist *)dev)->writer;
-    gx_clist_state * pcls = cdev->states; /* Use any. */
+    gx_device_clist * const cldev = (gx_device_clist *)dev;
+    gx_device_clist_writer * const cdev = &(cldev->writer);
+    gx_clist_state * pcls;
     int code;
 
+    /* flush previous contents */
+    if ((code = clist_close_writer_and_init_reader(cldev) < 0) ||
+        (code = clist_finish_page(dev, true)) < 0)
+        return code;;
+
+    pcls = cdev->states; /* Use any. */
+
     do {
         code = cmd_put_drawing_color(cdev, pcls, pdcolor, NULL, devn_not_tile);
         if (code >= 0)
@@ -328,12 +339,22 @@ clist_fill_rectangle(gx_device * dev, int rx, int ry, int rwidth, int rheight,
         return 0;
     if (cdev->permanent_error < 0)
       return (cdev->permanent_error);
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect bbox;
+
+        bbox.p.x = rx;
+        bbox.q.x = rx + rwidth - 1;
+        bbox.p.y = ry;
+        bbox.q.y = ry + rheight - 1;
+
+        clist_update_trans_bbox(cdev, &bbox);
+    }
     RECT_ENUM_INIT(re, ry, rheight);
     color_usage = gx_color_index2usage(dev, color);
     do {
         RECT_STEP_INIT(re);
         re.pcls->color_usage.or |= color_usage;
-        re.pcls->band_complexity.uses_color |= ((color_usage != 0) && (color_usage != gx_color_usage_all(cdev)));
         do {
             code = cmd_disable_lop(cdev, re.pcls);
             if (code >= 0 && color != re.pcls->colors[1]) {
@@ -368,25 +389,36 @@ clist_fill_rectangle_hl_color(gx_device *dev, const gs_fixed_rect *rect,
     int code;
     int rx, ry, rwidth, rheight;
     cmd_rects_enum_t re;
+    gx_color_usage_bits color_usage = cmd_drawing_color_usage(cdev, pdcolor);
 
-    rx = rect->p.x;
-    ry = rect->p.y;
-    rwidth = rect->q.x - rx;
-    rheight = rect->q.y - ry;
+    rx = fixed2int(rect->p.x);
+    ry = fixed2int(rect->p.y);
+    rwidth = fixed2int(rect->q.x) - rx;
+    rheight = fixed2int(rect->q.y) - ry;
 
     crop_fill(cdev, rx, ry, rwidth, rheight);
     if (rwidth <= 0 || rheight <= 0)
         return 0;
     if (cdev->permanent_error < 0)
       return (cdev->permanent_error);
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect bbox;
+
+        bbox.p.x = rx;
+        bbox.q.x = rx + rwidth - 1;
+        bbox.p.y = ry;
+        bbox.q.y = ry + rheight - 1;
+
+        clist_update_trans_bbox(cdev, &bbox);
+    }
     RECT_ENUM_INIT(re, ry, rheight);
     do {
         RECT_STEP_INIT(re);
-        re.pcls->color_usage.or = gx_color_usage_all(cdev);
-        re.pcls->band_complexity.uses_color = true;
+        re.pcls->color_usage.or |= color_usage;
         do {
             code = cmd_disable_lop(cdev, re.pcls);
-            code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re, 
+            code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re,
                                          devn_not_tile);
             if (code >= 0) {
                 code = cmd_write_rect_hl_cmd(cdev, re.pcls, cmd_op_fill_rect_hl,
@@ -418,14 +450,18 @@ clist_write_fill_trapezoid(gx_device * dev,
         &((gx_device_clist *)dev)->writer;
     int code;
     cmd_rects_enum_t re;
-    int ry, rheight;
+    int ry, rheight, rx, rxe;
     bool swap_axes = (options & 1);
 
     if (options & 4) {
         if (swap_axes) {
+            rx =  fixed2int(max(min(min(left->start.y, left->end.y), right->start.y), fa->clip->p.y));
+            rxe =  fixed2int_ceiling(min(max(max(left->start.y, left->end.y), right->start.y), fa->clip->q.y));
             ry = fixed2int(max(min(min(left->start.x, left->end.x), right->start.x), fa->clip->p.x));
             rheight = fixed2int_ceiling(min(max(max(left->start.x, left->end.x), right->start.x), fa->clip->q.x)) - ry;
         } else {
+            rx = fixed2int(max(min(min(left->start.x, left->end.x), right->start.x), fa->clip->p.x));
+            rxe = fixed2int_ceiling(min(max(max(left->start.x, left->end.x), right->start.x), fa->clip->q.x));
             ry = fixed2int(max(min(min(left->start.y, left->end.y), right->start.y), fa->clip->p.y));
             rheight = fixed2int_ceiling(min(max(max(left->start.y, left->end.y), right->start.y), fa->clip->q.y)) - ry;
         }
@@ -435,9 +471,13 @@ clist_write_fill_trapezoid(gx_device * dev,
            Would like to know a better range by X
            with computing intersections of sides with ybot, ytop. */
         if (swap_axes) {
+            rx = fixed2int(ybot);
+            rxe = fixed2int_ceiling(ytop);
             ry = fixed2int(min(left->start.x, left->end.x));
             rheight = fixed2int_ceiling(max(right->start.x, right->end.x)) - ry;
         } else {
+            rx = fixed2int(min(left->start.x, left->end.x));
+            rxe = fixed2int_ceiling(max(right->start.x, right->end.x));
             ry = fixed2int(ybot);
             rheight = fixed2int_ceiling(ytop) - ry;
         }
@@ -447,12 +487,23 @@ clist_write_fill_trapezoid(gx_device * dev,
         return 0;
     if (cdev->permanent_error < 0)
         return (cdev->permanent_error);
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect bbox;
+
+        bbox.p.x = rx;
+        bbox.q.x = rxe;
+        bbox.p.y = ry;
+        bbox.q.y = ry + rheight;
+
+        clist_update_trans_bbox(cdev, &bbox);
+    }
     RECT_ENUM_INIT(re, ry, rheight);
     do {
         RECT_STEP_INIT(re);
         do {
             if (pdcolor != NULL) {
-                code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re, 
+                code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re,
                                              devn_not_tile);
                 if (code == gs_error_unregistered)
                     return code;
@@ -550,19 +601,17 @@ clist_fill_linear_color_triangle(gx_device * dev, const gs_fill_attributes *fa,
     return 1;
 }
 
+extern dev_proc_open_device(pattern_clist_open_device);
+
 int
 clist_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
 {
-    gs_pattern1_instance_t *pinst = (gs_pattern1_instance_t *)data;
-    gx_bitmap_id id = (gx_bitmap_id)size;
     gx_device_clist_common * const cdev = &((gx_device_clist *)pdev)->common;
 
     if (dev_spec_op == gxdso_pattern_handles_clip_path)
         return 1;
     if (dev_spec_op == gxdso_pattern_shfill_doesnt_need_path)
         return 1;
-    if (dev_spec_op == gxdso_is_native_planar)
-        return cdev->is_planar;
     if (dev_spec_op == gxdso_supports_devn) {
         cmm_dev_profile_t *dev_profile;
         int code;
@@ -573,7 +622,10 @@ clist_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
             return 0;
         }
     }
-    return gx_default_dev_spec_op(pdev, dev_spec_op, pinst, id);
+    /* forward to our super class */
+    if (cdev->is_printer)
+        return gdev_prn_forwarding_dev_spec_op(pdev, dev_spec_op, data, size);
+    return gx_default_dev_spec_op(pdev, dev_spec_op, data, size);
 }
 
 #define dev_proc_pattern_manage(proc)\
@@ -583,7 +635,7 @@ clist_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
 int
 clist_strip_tile_rect_devn(gx_device * dev, const gx_strip_bitmap * tile,
                            int rx, int ry, int rwidth, int rheight,
-                           const gx_drawing_color *pdcolor0, 
+                           const gx_drawing_color *pdcolor0,
                            const gx_drawing_color *pdcolor1, int px, int py)
 {
     gx_device_clist_writer * const cdev =
@@ -591,19 +643,31 @@ clist_strip_tile_rect_devn(gx_device * dev, const gx_strip_bitmap * tile,
     int depth = 1;
     int code;
     cmd_rects_enum_t re;
+    gx_color_usage_bits color_usage = cmd_drawing_color_usage(cdev, pdcolor0);
 
     crop_fill(cdev, rx, ry, rwidth, rheight);
     if (rwidth <= 0 || rheight <= 0)
         return 0;
     if (cdev->permanent_error < 0)
       return (cdev->permanent_error);
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect bbox;
+
+        bbox.p.x = rx;
+        bbox.q.x = rx + rwidth - 1;
+        bbox.p.y = ry;
+        bbox.q.y = ry + rheight - 1;
+
+        clist_update_trans_bbox(cdev, &bbox);
+    }
+    color_usage |= cmd_drawing_color_usage(cdev, pdcolor1);
     RECT_ENUM_INIT(re, ry, rheight);
     do {
         ulong offset_temp;
 
         RECT_STEP_INIT(re);
-        re.pcls->color_usage.or = gx_color_usage_all(cdev);
-        re.pcls->band_complexity.uses_color = true;
+        re.pcls->color_usage.or |= color_usage;
         do {
             code = cmd_disable_lop(cdev, re.pcls);
         } while (RECT_RECOVER(code));
@@ -618,17 +682,17 @@ clist_strip_tile_rect_devn(gx_device * dev, const gx_strip_bitmap * tile,
                 if (code < 0 && !(code != gs_error_VMerror || !cdev->error_is_retryable) && SET_BAND_CODE(code))
                     goto error_in_rect;
             } else
-                code = -1; 
+                code = -1;
             if (code < 0) {
-                return_error(gs_error_unregistered); 
+                return_error(gs_error_unregistered);
             }
         }
         do {
             code = 0;
             /* Write out the devn colors */
-            code = cmd_put_drawing_color(cdev, re.pcls, pdcolor0, &re, 
+            code = cmd_put_drawing_color(cdev, re.pcls, pdcolor0, &re,
                                          devn_tile0);
-            code = cmd_put_drawing_color(cdev, re.pcls, pdcolor1, &re, 
+            code = cmd_put_drawing_color(cdev, re.pcls, pdcolor1, &re,
                                          devn_tile1);
             /* Set the tile phase */
             if (px != re.pcls->tile_phase.x || py != re.pcls->tile_phase.y) {
@@ -637,7 +701,7 @@ clist_strip_tile_rect_devn(gx_device * dev, const gx_strip_bitmap * tile,
             }
             /* Write out the actually command to fill with the devn colors */
             if (code >= 0) {
-                code = cmd_write_rect_hl_cmd(cdev, re.pcls, 
+                code = cmd_write_rect_hl_cmd(cdev, re.pcls,
                                              cmd_opv_ext_tile_rect_hl, rx, re.y,
                                              rwidth, re.height, true);
             }
@@ -677,17 +741,24 @@ clist_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tile,
     if (rwidth <= 0 || rheight <= 0)
         return 0;
     if (cdev->permanent_error < 0)
-      return (cdev->permanent_error);
+        return (cdev->permanent_error);
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect bbox;
+
+        bbox.p.x = rx;
+        bbox.q.x = rx + rwidth - 1;
+        bbox.p.y = ry;
+        bbox.q.y = ry + rheight - 1;
+
+        clist_update_trans_bbox(cdev, &bbox);
+    }
     RECT_ENUM_INIT(re, ry, rheight);
     do {
         ulong offset_temp;
 
         RECT_STEP_INIT(re);
         re.pcls->color_usage.or |= color_usage;
-        re.pcls->band_complexity.uses_color |=
-            ((color0 != gx_no_color_index) && (color0 != 0xffffff) && (color0 != 0)) ||
-            ((color1 != gx_no_color_index) && (color1 != 0xffffff) && (color1 != 0));
-
         do {
             code = cmd_disable_lop(cdev, re.pcls);
         } while (RECT_RECOVER(code));
@@ -752,15 +823,23 @@ clist_copy_mono(gx_device * dev,
     gx_color_usage_bits color_usage =
         (color0 == gx_no_color_index ? 0 : gx_color_index2usage(dev, color0)) |
         (color1 == gx_no_color_index ? 0 : gx_color_index2usage(dev, color1));
-    bool uses_color =
-        ((color0 != gx_no_color_index) && (color0 != 0xffffff) && (color0 != 0)) ||
-        ((color1 != gx_no_color_index) && (color1 != 0xffffff) && (color1 != 0));
     cmd_rects_enum_t re;
 
     fit_copy(dev, data, data_x, raster, id, rx, ry, rwidth, rheight);
     y0 = ry;
     if (cdev->permanent_error < 0)
       return (cdev->permanent_error);
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect bbox;
+
+        bbox.p.x = rx;
+        bbox.q.x = rx + rwidth - 1;
+        bbox.p.y = ry;
+        bbox.q.y = ry + rheight - 1;
+
+        clist_update_trans_bbox(cdev, &bbox);
+    }
     RECT_ENUM_INIT(re, ry, rheight);
     do {
         int dx = data_x & 7;
@@ -770,7 +849,6 @@ clist_copy_mono(gx_device * dev,
 
         RECT_STEP_INIT(re);
         re.pcls->color_usage.or |= color_usage;
-        re.pcls->band_complexity.uses_color |= uses_color;
         do {
             code = cmd_disable_lop(cdev, re.pcls);
             if (code >= 0)
@@ -838,10 +916,10 @@ copy:{
         }
         op += compress;
         if (dx) {
-            *dp++ = cmd_count_op(cmd_opv_set_misc, 2);
+            *dp++ = cmd_count_op(cmd_opv_set_misc, 2, dev->memory);
             *dp++ = cmd_set_misc_data_x + dx;
         }
-        *dp++ = cmd_count_op(op, csize);
+        *dp++ = cmd_count_op(op, csize, dev->memory);
         /* Store the plane_height */
         cmd_putw(0, dp);
         cmd_put2w(rx, re.y, dp);
@@ -879,13 +957,38 @@ clist_copy_planes(gx_device * dev,
     y0 = ry;
     if (cdev->permanent_error < 0)
       return (cdev->permanent_error);
+
+#ifdef DEBUG
+    if (plane_height == 0) {
+        dprintf("clist_copy_planes called with plane_height == 0.\n");
+    }
+#endif
+
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect bbox;
+
+        bbox.p.x = rx;
+        bbox.q.x = rx + rwidth - 1;
+        bbox.p.y = ry;
+        bbox.q.y = ry + rheight - 1;
+
+        clist_update_trans_bbox(cdev, &bbox);
+    }
+
     RECT_ENUM_INIT(re, ry, rheight);
     do {
+        int code;
+        gx_cmd_rect rect;
+        int rsize;
+        byte *dp, *dp2;
+        uint csize;
+        byte op = (byte) cmd_op_copy_mono_planes;
         int dx = data_x % (8/bpc);
         int w1 = dx + rwidth;
         const byte *row = data + (re.y - y0) * raster + (data_x / (8/bpc));
-        int code;
-
+        int bytes_row = ((w1*bpc+7)/8 + 7) & ~7;
+        int maxheight = data_bits_size / bytes_row / cdev->color_info.num_components;
 
         RECT_STEP_INIT(re);
         do {
@@ -897,44 +1000,35 @@ clist_copy_planes(gx_device * dev,
             goto error_in_rect;
         /* Don't bother to check for a possible cache hit: */
         /* tile_rectangle and fill_mask handle those cases. */
-copy:
-        /* We require that in the copy_planes case all the planes fit into
-         * a single cbuf. */
-        if (plane_height > 0) {
-            int bytes_row = ((w1*bpc+7)/8 + 7) & ~7;
-            int maxheight = (cbuf_size - 0x100) / bytes_row / cdev->color_info.num_components;
-            if (re.height > maxheight)
-                re.height = maxheight;
-            if (re.height == 0)
-            {
-                /* Split a single (very long) row. */
-                int w2 = w1 >> 1;
 
-		re.height = 1;
-                ++cdev->driver_call_nesting;
-                {
-                    code = clist_copy_planes(dev, row, dx, raster,
-                                             gx_no_bitmap_id, rx, re.y,
-                                             w2, 1, plane_height);
-                    if (code >= 0)
-                        code = clist_copy_planes(dev, row, dx + w2,
-                                                 raster, gx_no_bitmap_id,
-                                                 rx + w2, re.y,
-                                                 w1 - w2, 1, plane_height);
-                }
-                --cdev->driver_call_nesting;
-                if (code < 0 && SET_BAND_CODE(code))
-                    goto error_in_rect;
-                continue;
-            }
+        /* We require that in the copy_planes case all the planes fit into
+         * the data_bits area (for the copy_planes call by the clist reader) */
+        if (re.height > maxheight)
+            re.height = maxheight;
+
+        if (re.height == 0) {
+            /* Not even a single row fits. Revert to a single row and split it in two recursively */
+            int w2 = w1 >> 1;
+
+            re.height = 1;
+            ++cdev->driver_call_nesting;
+            code = clist_copy_planes(dev, row, dx, raster,
+                                     gx_no_bitmap_id, rx, re.y,
+                                     w2, 1, plane_height);
+            if (code >= 0)
+                code = clist_copy_planes(dev, row, dx + w2,
+                                         raster, gx_no_bitmap_id,
+                                         rx + w2, re.y,
+                                         w1 - w2, 1, plane_height);
+            --cdev->driver_call_nesting;
+            if (code < 0 && SET_BAND_CODE(code))
+                goto error_in_rect;
+            continue;
         }
-	{
-        gx_cmd_rect rect;
-        int rsize;
-        byte op = (byte) cmd_op_copy_mono_planes;
-        byte *dp, *dp2;
-        uint csize;
-        int code;
+
+        /* 0x100 fudge is arbitrary, but the BufferSpace is large w.r.t. cbuf size so it doesn't matter */
+        if ((cdev->cend - cdev->cnext) < 0x100 + (re.height * bytes_row * cdev->color_info.num_components))
+            cmd_write_buffer(cdev, cmd_opv_end_run);	/* Insure that all planes fit in the bufferspace */
 
         rect.x = rx, rect.y = re.y;
         rect.width = w1, rect.height = re.height;
@@ -953,70 +1047,37 @@ copy:
              * cmd_put_bits fill the buffer up. */
             dp2 = dp;
             if (dx) {
-                *dp2++ = cmd_count_op(cmd_opv_set_misc, 2);
+                *dp2++ = cmd_count_op(cmd_opv_set_misc, 2, cdev->memory);
                 *dp2++ = cmd_set_misc_data_x + dx;
             }
-            *dp2++ = cmd_count_op(op + code, csize);
+            *dp2++ = cmd_count_op(op + code, csize, cdev->memory);
             cmd_putw(plane_height, dp2);
             cmd_put2w(rx, re.y, dp2);
             cmd_put2w(w1, re.height, dp2);
-            if (plane_height > 0) {
-                for (plane = 1; plane < cdev->color_info.num_components && (code >= 0); plane++)
-                {
-                    byte *dummy_dp;
-                    uint csize2;
-                    /* Copy subsequent planes - 1 byte header used to send the
-                     * compression type. */
-                    code = cmd_put_bits(cdev, re.pcls,
-                                        row + plane_height * raster * plane,
-                                        w1*bpc, re.height, raster, 1,
-                                        (bpc == 1 ?
-                                         (orig_id == gx_no_bitmap_id ?
-                                          1 << cmd_compress_rle :
-                                          cmd_mask_compress_any) : 0),
-                                        &dummy_dp, &csize2);
-                    if (code >= 0)
-                        *dummy_dp = code;
+            for (plane = 1; plane < cdev->color_info.num_components && (code >= 0); plane++)
+            {
+                byte *dummy_dp;
+                uint csize2;
+                /* Copy subsequent planes - 1 byte header used to send the
+                 * compression type. */
+                code = cmd_put_bits(cdev, re.pcls,
+                                    row + plane_height * raster * plane,
+                                    w1*bpc, re.height, raster, 1,
+                                    (bpc == 1 ?
+                                     (orig_id == gx_no_bitmap_id ?
+                                      1 << cmd_compress_rle :
+                                      cmd_mask_compress_any) : 0),
+                                    &dummy_dp, &csize2);
+                if (code >= 0)
+                    *dummy_dp = code;
 
-                    csize += csize2;
-                }
+                csize += csize2;
             }
         } while (RECT_RECOVER(code));
         if (code < 0 && !(code == gs_error_limitcheck) && SET_BAND_CODE(code))
             goto error_in_rect;
-        if (code < 0) {
-            /* The bitmap was too large; split up the transfer. */
-            if (re.height > 1) {
-                /*
-                 * Split the transfer by reducing the height.
-                 * See the comment above FOR_RECTS in gxcldev.h.
-                 */
-                re.height >>= 1;
-                goto copy;
-            } else {
-                /* Split a single (very long) row. */
-                int w2 = w1 >> 1;
-
-                ++cdev->driver_call_nesting;
-                {
-                    code = clist_copy_planes(dev, row, dx, raster,
-                                             gx_no_bitmap_id, rx, re.y,
-                                             w2, 1, plane_height);
-                    if (code >= 0)
-                        code = clist_copy_planes(dev, row, dx + w2,
-                                                 raster, gx_no_bitmap_id,
-                                                 rx + w2, re.y,
-                                                 w1 - w2, 1, plane_height);
-                }
-                --cdev->driver_call_nesting;
-                if (code < 0 && SET_BAND_CODE(code))
-                    goto error_in_rect;
-                continue;
-            }
-        }
 
         re.pcls->rect = rect;
-        }
         continue;
 error_in_rect:
         if (!(cdev->error_is_retryable && cdev->driver_call_nesting == 0 &&
@@ -1047,6 +1108,17 @@ clist_copy_color(gx_device * dev,
     data_x_bit = data_x * depth;
     if (cdev->permanent_error < 0)
       return (cdev->permanent_error);
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect bbox;
+
+        bbox.p.x = rx;
+        bbox.q.x = rx + rwidth - 1;
+        bbox.p.y = ry;
+        bbox.q.y = ry + rheight - 1;
+
+        clist_update_trans_bbox(cdev, &bbox);
+    }
     RECT_ENUM_INIT(re, ry, rheight);
     do {
         int dx = (data_x_bit & 7) / depth;
@@ -1056,7 +1128,6 @@ clist_copy_color(gx_device * dev,
 
         RECT_STEP_INIT(re);
         re.pcls->color_usage.or |= all;
-        re.pcls->band_complexity.uses_color = 1;
 
         do {
             code = cmd_disable_lop(cdev, re.pcls);
@@ -1125,10 +1196,10 @@ copy:{
             }
             op += compress;
             if (dx) {
-                *dp++ = cmd_count_op(cmd_opv_set_misc, 2);
+                *dp++ = cmd_count_op(cmd_opv_set_misc, 2, dev->memory);
                 *dp++ = cmd_set_misc_data_x + dx;
             }
-            *dp++ = cmd_count_op(op, csize);
+            *dp++ = cmd_count_op(op, csize, dev->memory);
             cmd_put2w(rx, re.y, dp);
             cmd_put2w(w1, re.height, dp);
             re.pcls->rect = rect;
@@ -1143,10 +1214,10 @@ error_in_rect:
     return 0;
 }
 
-/* Patterned after clist_copy_alpha and sharing a command with 
+/* Patterned after clist_copy_alpha and sharing a command with
    cmd_op_copy_color_alpha.  This was done due to avoid difficult
-   to follow code in the read back logic in gxclrast.c.  Due to the 
-   recursive nature of this call, it would be a bit painful to do much 
+   to follow code in the read back logic in gxclrast.c.  Due to the
+   recursive nature of this call, it would be a bit painful to do much
    sharing between clist_copy_alpha_hl_color and clist_copy_alpha */
 int
 clist_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
@@ -1171,6 +1242,17 @@ clist_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
     data_x_bit = data_x << log2_depth;
     if (cdev->permanent_error < 0)
       return (cdev->permanent_error);
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect bbox;
+
+        bbox.p.x = rx;
+        bbox.q.x = rx + rwidth - 1;
+        bbox.p.y = ry;
+        bbox.q.y = ry + rheight - 1;
+
+        clist_update_trans_bbox(cdev, &bbox);
+    }
     RECT_ENUM_INIT(re, ry, rheight);
     do {
         int dx = (data_x_bit & 7) >> log2_depth;
@@ -1180,7 +1262,6 @@ clist_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
 
         RECT_STEP_INIT(re);
         re.pcls->color_usage.or = all;
-        re.pcls->band_complexity.uses_color = true;
         do {
             code = cmd_disable_lop(cdev, re.pcls);
             if (code >= 0)
@@ -1214,7 +1295,7 @@ clist_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
         }
         /* Set the color */
         do {
-            code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re, 
+            code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re,
                                          devn_not_tile);
         } while (RECT_RECOVER(code));
 copy:{
@@ -1267,10 +1348,10 @@ copy:{
             }
             op += compress;
             if (dx) {
-                *dp++ = cmd_count_op(cmd_opv_set_misc, 2);
+                *dp++ = cmd_count_op(cmd_opv_set_misc, 2, dev->memory);
                 *dp++ = cmd_set_misc_data_x + dx;
             }
-            *dp++ = cmd_count_op(op, csize);
+            *dp++ = cmd_count_op(op, csize, dev->memory);
             *dp++ = depth;
             cmd_put2w(rx, re.y, dp);
             cmd_put2w(w1, re.height, dp);
@@ -1312,6 +1393,17 @@ clist_copy_alpha(gx_device * dev, const byte * data, int data_x,
     data_x_bit = data_x << log2_depth;
     if (cdev->permanent_error < 0)
       return (cdev->permanent_error);
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect bbox;
+
+        bbox.p.x = rx;
+        bbox.q.x = rx + rwidth - 1;
+        bbox.p.y = ry;
+        bbox.q.y = ry + rheight - 1;
+
+        clist_update_trans_bbox(cdev, &bbox);
+    }
     RECT_ENUM_INIT(re, ry, rheight);
     do {
         int dx = (data_x_bit & 7) >> log2_depth;
@@ -1344,7 +1436,7 @@ clist_copy_alpha(gx_device * dev, const byte * data, int data_x,
 
             do {
                 code = set_cmd_put_op(dp, cdev, re.pcls, cmd_opv_extend, 1);
-                code = set_cmd_put_op(dp, cdev, re.pcls, 
+                code = set_cmd_put_op(dp, cdev, re.pcls,
                                       cmd_opv_ext_unset_color_is_devn, 1);
             } while (RECT_RECOVER(code));
             if (code < 0 && SET_BAND_CODE(code))
@@ -1416,10 +1508,10 @@ copy:{
             }
             op += compress;
             if (dx) {
-                *dp++ = cmd_count_op(cmd_opv_set_misc, 2);
+                *dp++ = cmd_count_op(cmd_opv_set_misc, 2, dev->memory);
                 *dp++ = cmd_set_misc_data_x + dx;
             }
-            *dp++ = cmd_count_op(op, csize);
+            *dp++ = cmd_count_op(op, csize, dev->memory);
             *dp++ = depth;
             cmd_put2w(rx, re.y, dp);
             cmd_put2w(w1, re.height, dp);
@@ -1483,6 +1575,18 @@ clist_strip_copy_rop2(gx_device * dev,
     }
     if (rwidth <= 0 || rheight <= 0)
         return 0;
+
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect bbox;
+
+        bbox.p.x = rx;
+        bbox.q.x = rx + rwidth - 1;
+        bbox.p.y = ry;
+        bbox.q.y = ry + rheight - 1;
+
+        clist_update_trans_bbox(cdev, &bbox);
+    }
     /*
      * On CMYK devices, RasterOps must be executed with complete pixels
      * if the operation involves the destination.
@@ -1508,7 +1612,16 @@ clist_strip_copy_rop2(gx_device * dev,
     y0 = ry;
     if (cdev->permanent_error < 0)
       return (cdev->permanent_error);
-    {
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect bbox;
+
+        bbox.p.x = rx;
+        bbox.q.x = rx + rwidth - 1;
+        bbox.p.y = ry;
+        bbox.q.y = ry + rheight - 1;
+
+        clist_update_trans_bbox(cdev, &bbox);
     }
     RECT_ENUM_INIT(re, ry, rheight);
     do {
@@ -1530,12 +1643,15 @@ clist_strip_copy_rop2(gx_device * dev,
             D = (rop_operand)re.pcls->color_usage.or;
             re.pcls->color_usage.or |=
                     ((rop_proc_table[color_rop])(D, (rop_operand)S, (rop_operand)T) & all);
-            D = (rop_operand)(re.pcls->color_usage.or>>(8*sizeof(rop_operand)));
+#define DOWN(x) ((rop_operand)(((x)>>(4*sizeof(rop_operand)))>>(4*sizeof(rop_operand))))
+#define UP(x)   ((((gx_color_usage_bits)(x))<<(4*sizeof(rop_operand)))<<(4*sizeof(rop_operand)))
+            D = DOWN(re.pcls->color_usage.or);
             re.pcls->color_usage.or |=
-                    (((gx_color_usage_bits)(rop_proc_table[color_rop])(D, (rop_operand)(S>>(8*sizeof(rop_operand))), (rop_operand)(T>>(8*sizeof(rop_operand)))))<<(8*sizeof(rop_operand))) & all;
+                    UP((rop_proc_table[color_rop])(D, DOWN(S), DOWN(T))) & all;
+#undef DOWN
+#undef UP
         }
         re.pcls->color_usage.slow_rop |= slow_rop;
-        re.pcls->band_complexity.nontrivial_rops |= slow_rop;
         if (rop3_uses_T(rop)) {
             if (tcolors == 0 || tcolors[0] != tcolors[1]) {
                 ulong offset_temp;
