@@ -102,7 +102,7 @@
  * 1 branch on the no-step case.
  *
  * This is an even better win on architectures (like ARM) where alternation
- * can be done without branches:
+ * can be done without branches; the original gives:
  *
  *   ADD   Q, Q, dQ
  *   CMP   R, dR
@@ -110,7 +110,7 @@
  *   ADDLE Q, Q, #1
  *   ADDLE R, R, nDR
  *
- * in the original (assuming all values in registers) becomes:
+ * (assuming all values in registers) and the new formulation becomes:
  *
  *   SUBS  R, R, dR
  *   ADDLT R, R, N
@@ -125,123 +125,9 @@
  *   ADC   Q, Q, dQ
  *
  * (Actually looking at the compilation given by MSVC 2005 confirms a 1
- * instruction (and once branch) win for the above results. Sadly compilers
+ * instruction (and one branch) win for the above results. Sadly compilers
  * don't look to be smart enough to exploit carry flag tricks.)
  */
-#ifdef USE_OLD_DDA_FORMULATION
-
-/* In the following structure definitions, ntype must be an unsigned type. */
-#define dda_state_struct(sname, dtype, ntype)\
-  struct sname { dtype Q; ntype R; }
-#define dda_step_struct(sname, dtype, ntype)\
-  struct sname { dtype dQ; ntype dR, NdR; }
-
-/* DDA with int Q and uint N */
-typedef struct gx_dda_int_s {
-    dda_state_struct(ia_, int, uint) state;
-    dda_step_struct(ie_, int, uint) step;
-} gx_dda_int_t;
-
-/* DDA with fixed Q and (unsigned) integer N */
-typedef
-dda_state_struct(_a, fixed, uint) gx_dda_state_fixed;
-     typedef dda_step_struct(_e, fixed, uint) gx_dda_step_fixed;
-     typedef struct gx_dda_fixed_s {
-         gx_dda_state_fixed state;
-         gx_dda_step_fixed step;
-     } gx_dda_fixed;
-/*
- * Define a pair of DDAs for iterating along an arbitrary line.
- */
-     typedef struct gx_dda_fixed_point_s {
-         gx_dda_fixed x, y;
-     } gx_dda_fixed_point;
-/*
- * Initialize a DDA.  The sign test is needed only because C doesn't
- * provide reliable definitions of / and % for integers (!!!).
- */
-#define dda_init_state(dstate, init, N)\
-  (dstate).Q = (init), (dstate).R = (N)
-#define dda_init_step(dstep, D, N)\
-  if ( (N) == 0 )\
-    (dstep).dQ = 0, (dstep).dR = 0;\
-  else if ( (D) < 0 )\
-   { (dstep).dQ = -(int)((uint)-(D) / (N));\
-     if ( ((dstep).dR = -(D) % (N)) != 0 )\
-       --(dstep).dQ, (dstep).dR = (N) - (dstep).dR;\
-   }\
-  else\
-   { (dstep).dQ = (D) / (N); (dstep).dR = (D) % (N); }\
-  (dstep).NdR = (N) - (dstep).dR
-#define dda_init(dda, init, D, N)\
-  dda_init_state((dda).state, init, N);\
-  dda_init_step((dda).step, D, N)
-/*
- * Compute the sum of two DDA steps with the same D and N.
- * Note that since dR + NdR = N, this quantity must be the same in both
- * fromstep and tostep.
- */
-#define dda_step_add(tostep, fromstep)\
-  (tostep).dQ +=\
-    ((tostep).dR < (fromstep).NdR ?\
-     ((tostep).dR += (fromstep).dR, (tostep).NdR -= (fromstep).dR,\
-      (fromstep).dQ) :\
-     ((tostep).dR -= (fromstep).NdR, (tostep).NdR += (fromstep).NdR,\
-      (fromstep).dQ + 1))
-/*
- * Return the current value in a DDA.
- */
-#define dda_state_current(dstate) (dstate).Q
-#define dda_current(dda) dda_state_current((dda).state)
-#define dda_current_fixed2int(dda)\
-  fixed2int_var(dda_state_current((dda).state))
-/*
- * Increment a DDA to the next point.
- * Returns the updated current value.
- */
-#define dda_state_next(dstate, dstep)\
-  (dstate).Q +=\
-    ((dstate).R > (dstep).dR ?\
-     ((dstate).R -= (dstep).dR, (dstep).dQ) :\
-     ((dstate).R += (dstep).NdR, (dstep).dQ + 1))
-#define dda_next(dda) dda_state_next((dda).state, (dda).step)
-#define dda_next_assign(dda,v) \
-                            do { dda_next(dda);(v)=(dda).state.Q; } while (0)
-/*
- * Back up a DDA to the previous point.
- * Returns the updated current value.
- */
-#define dda_state_previous(dstate, dstep)\
-  (dstate).Q -=\
-    ((dstate).R <= (dstep).NdR ?\
-     ((dstate).R += (dstep).dR, (dstep).dQ) :\
-     ((dstate).R -= (dstep).NdR, (dstep).dQ + 1))
-#define dda_previous(dda) dda_state_previous((dda).state, (dda).step)
-#define dda_previous_assign(dda,v) \
-                         do { dda_previous(dda);v =(dda).state.Q; } while (0)
-/*
- * Advance a DDA by an arbitrary number of steps.
- * This implementation is very inefficient; we'll improve it if needed.
- */
-#define dda_state_advance(dstate, dstep, nsteps)\
-  BEGIN\
-    uint n_ = (nsteps);\
-    (dstate).Q += (dstep).dQ * (nsteps);\
-    while ( n_-- )\
-      if ( (dstate).R > (dstep).dR ) (dstate).R -= (dstep).dR;\
-      else (dstate).R += (dstep).NdR, (dstate).Q++;\
-  END
-#define dda_advance(dda, nsteps)\
-  dda_state_advance((dda).state, (dda).step, nsteps)
-/*
- * Translate the position of a DDA by a given amount.
- */
-#define dda_state_translate(dstate, delta)\
-  ((dstate).Q += (delta))
-#define dda_translate(dda, delta)\
-  dda_state_translate((dda).state, delta)
-
-#else /* New Version! */
 
 /* In the following structure definitions, ntype must be an unsigned type. */
 #define dda_state_struct(sname, dtype, ntype)\
@@ -289,8 +175,27 @@ dda_state_struct(_a, fixed, uint) gx_dda_state_fixed;
     (dstep).N = (N_);\
   } while (0)
 #define dda_init(dda, init, D, N)\
-  dda_init_state((dda).state, init, N);\
-  dda_init_step((dda).step, D, N)
+  do {\
+    dda_init_state((dda).state, init, N);\
+    dda_init_step((dda).step, D, N);\
+  } while (0)
+
+/*
+ * Initialise a DDA, and do a half step.
+ */
+#define dda_init_state_half(dstate, dstep, init, N_)\
+  (dstate).Q = (init) + ((dstep).dQ>>1);\
+  (dstate).R = ((N_)-1) - (((dstep).dR + ((dstep).dQ & 1 ? (dstep).N : 0))>>1);\
+  if ((signed)(dstate).R < 0) {\
+    (dstate).Q++;\
+    (dstate).R += (dstep).N;\
+  }
+#define dda_init_half(dda, init, D, N)\
+  do {\
+    dda_init_step((dda).step, D, N);\
+    dda_init_state_half((dda).state, (dda).step, init, N);\
+  } while (0)
+
 /*
  * Compute the sum of two DDA steps with the same D and N.
  * Note that since dR + NdR = N, this quantity must be the same in both
@@ -369,6 +274,4 @@ dda_state_struct(_a, fixed, uint) gx_dda_state_fixed;
 #define dda_translate(dda, delta)\
   dda_state_translate((dda).state, delta)
 
-#endif
-
 #endif /* gxdda_INCLUDED */
