@@ -43,8 +43,10 @@
 #ifdef __MACH__
 #include <string.h>
 #else
+#ifndef memset
 void *memset(void *,int,size_t);
 #endif
+#endif
 int atexit(void (*)(void));
 #else
 #include "memento.h"
@@ -523,6 +525,7 @@ int Memento_listBlocksNested(void)
     count = 0;
     size = 0;
     for (b = globals.used.head; b; b = b->next) {
+        VALGRIND_MAKE_MEM_DEFINED(b, sizeof(*b));
         size += b->rawsize;
         count++;
     }
@@ -602,6 +605,14 @@ int Memento_listBlocksNested(void)
     fprintf(stderr, " Total size of blocks = %d\n", size);
 
     MEMENTO_UNDERLYING_FREE(blocks);
+
+    /* Now put the blocks back for valgrind */
+    for (b = globals.used.head; b;) {
+      Memento_BlkHeader *next = b->next;
+      VALGRIND_MAKE_MEM_NOACCESS(b, sizeof(*b));
+      b = next;
+    }
+
     return 0;
 }
 
@@ -899,7 +910,9 @@ void *Memento_label(void *ptr, const char *label)
     if (ptr == NULL)
         return NULL;
     block = MEMBLK_FROMBLK(ptr);
+    VALGRIND_MAKE_MEM_DEFINED(&block->label, sizeof(block->label));
     block->label = label;
+    VALGRIND_MAKE_MEM_NOACCESS(&block->label, sizeof(block->label));
     return ptr;
 }
 
@@ -997,6 +1010,7 @@ static int checkBlock(Memento_BlkHeader *memblk, const char *action)
         /* Failure! */
         fprintf(stderr, "Attempt to %s block ", action);
         showBlock(memblk, 32);
+        fprintf(stderr, "\n");
         Memento_breakpoint();
         return 1;
     } else if (data.preCorrupt || data.postCorrupt) {
@@ -1035,10 +1049,10 @@ void Memento_free(void *blk)
     if (checkBlock(memblk, "free"))
         return;
 
+    VALGRIND_MAKE_MEM_DEFINED(memblk, sizeof(*memblk));
     if (memblk->flags & Memento_Flag_BreakOnFree)
         Memento_breakpoint();
 
-    VALGRIND_MAKE_MEM_DEFINED(memblk, sizeof(*memblk));
     globals.alloc -= memblk->rawsize;
     globals.numFrees++;
 
