@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* DCT filter parameter setting and reading */
 #include "memory_.h"
 #include "jpeglib_.h"
@@ -84,7 +86,7 @@ static const byte inverse_natural_order[DCTSIZE2] =
 
 static int
 quant_param_string(gs_param_string * pstr, int count, const UINT16 * pvals,
-                   floatp QFactor, gs_memory_t * mem)
+                   double QFactor, gs_memory_t * mem)
 {
     byte *data;
     int code = 0;
@@ -94,7 +96,7 @@ quant_param_string(gs_param_string * pstr, int count, const UINT16 * pvals,
     if (data == 0)
         return_error(gs_error_VMerror);
     for (i = 0; i < count; ++i) {
-        floatp val = pvals[jpeg_inverse_order(i)] / QFactor;
+        double val = pvals[jpeg_inverse_order(i)] / QFactor;
 
         data[i] =
             (val < 1 ? (code = 1) : val > 255 ? (code = 255) : (byte) val);
@@ -107,7 +109,7 @@ quant_param_string(gs_param_string * pstr, int count, const UINT16 * pvals,
 
 static int
 quant_param_array(gs_param_float_array * pfa, int count, const UINT16 * pvals,
-                  floatp QFactor, gs_memory_t * mem)
+                  double QFactor, gs_memory_t * mem)
 {
     float *data;
     int i;
@@ -138,7 +140,7 @@ s_DCT_get_quantization_tables(gs_param_list * plist,
     JQUANT_TBL **table_ptrs;
     JQUANT_TBL **default_table_ptrs;
     gs_param_array quant_tables;
-    floatp QFactor = pdct->QFactor;
+    double QFactor = pdct->QFactor;
     int i;
     int code;
 
@@ -198,7 +200,7 @@ s_DCT_get_quantization_tables(gs_param_list * plist,
         gs_param_string str;
         gs_param_float_array fa;
 
-        sprintf(key, "%d", i);
+        gs_sprintf(key, "%d", i);
         if (QFactor == 1.0) {
             code = quant_param_string(&str, DCTSIZE2,
                             table_ptrs[comp_info[i].quant_tbl_no]->quantval,
@@ -368,7 +370,7 @@ s_DCT_byte_params(gs_param_list * plist, gs_param_name key, int start,
 /* Get N quantization values from an array or a string. */
 static int
 quant_params(gs_param_list * plist, gs_param_name key, int count,
-             UINT16 * pvals, floatp QFactor)
+             UINT16 * pvals, double QFactor)
 {
     int i;
     gs_param_string bytes;
@@ -472,7 +474,7 @@ s_DCT_put_quantization_tables(gs_param_list * plist, stream_DCT_state * pdct,
         char istr[5];		/* i converted to string key */
         UINT16 values[DCTSIZE2];
 
-        sprintf(istr, "%d", i);
+        gs_sprintf(istr, "%d", i);
         code = quant_params(quant_tables.list, istr, DCTSIZE2, values,
                             pdct->QFactor);
         if (code < 0)
@@ -564,7 +566,7 @@ s_DCT_put_huffman_tables(gs_param_list * plist, stream_DCT_state * pdct,
         UINT8 counts[16], values[256];
 
         /* Collect the Huffman parameters. */
-        sprintf(istr, "%d", i);
+        gs_sprintf(istr, "%d", i);
         code = s_DCT_byte_params(huff_tables.list, istr, 0, 16, counts);
         if (code < 0)
             return code;
