@@ -422,7 +422,7 @@ calculate_dst_contrib(stream_IScale_state * ss, int y)
                       ss->filter, ss->min_scale);
     int first_index_mod = ss->dst_next_list.first_pixel / row_size;
 
-    if_debug2('w', "[W]calculate_dst_contrib for y = %d, y+offset=%d\n", y, y + ss->src_y_offset);
+    if_debug2m('w', ss->memory, "[W]calculate_dst_contrib for y = %d, y+offset=%d\n", y, y + ss->src_y_offset);
     ss->dst_last_index = last_index;
     last_index %= ss->max_support;
     if (last_index < first_index_mod) {         /* Shuffle the indices to account for wraparound. */
@@ -436,13 +436,13 @@ calculate_dst_contrib(stream_IScale_state * ss, int y)
                  i >= first_index_mod ?
                  ss->dst_items[i - first_index_mod].weight :
                  0);
-            if_debug1('W', " %f", shuffle[i].weight);
+            if_debug1m('W', ss->memory, " %f", shuffle[i].weight);
         }
         memcpy(ss->dst_items, shuffle, ss->max_support * sizeof(CONTRIB));
         ss->dst_next_list.n = ss->max_support;
         ss->dst_next_list.first_pixel = 0;
     }
-    if_debug0('W', "\n");
+    if_debug0m('W', ss->memory, "\n");
 }
 
 /* Set default parameter values (actually, just clear pointers). */
@@ -528,6 +528,16 @@ do_init(stream_state        *st,
         return ERRC;
 /****** WRONG ******/
     }
+#ifdef PACIFY_VALGRIND
+    /* When we are scaling a subrectangle of an image, we calculate
+     * the subrectangle, so that it's slightly larger than it needs
+     * to be. Some of these 'extra' pixels are calculated using
+     * bogus values (i.e. ones we don't bother copying/scaling into
+     * the line buffer). These cause valgrind to be upset. To avoid
+     * this, we preset the buffer to known values. */
+    memset((byte *)ss->tmp, 0,
+           ss->max_support * ss->params.WidthOut * ss->params.spp_interp);
+#endif
     /* Pre-calculate filter contributions for a row. */
     calculate_contrib(ss->contrib, ss->items,
                       (double)ss->params.EntireWidthOut / ss->params.EntireWidthIn,
@@ -659,8 +669,9 @@ s_IScale_process(stream_state * st, stream_cursor_read * pr,
                 row = pr->ptr + 1;
             } else {            /* We're buffering a row in src. */
                 row = ss->src;
-                memcpy((byte *) ss->src + ss->src_offset, pr->ptr + 1,
-                       rcount);
+                if (ss->params.Active)
+                    memcpy((byte *) ss->src + ss->src_offset, pr->ptr + 1,
+                           rcount);
                 ss->src_offset = 0;
             }
             /* Apply filter to zoom horizontally from src to tmp. */
