@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Generic substitute for Unix sys/time.h */
 
 #ifndef time__INCLUDED
@@ -31,7 +33,7 @@
  * Some System V environments don't include sys/time.h.
  * The HAVE_SYS_TIME_H switch in gconfig_.h reflects this.
  */
-#ifdef HAVE_SYS_TIME_H
+#if defined(HAVE_SYS_TIME_H) && HAVE_SYS_TIME_H == 1
 #  include <sys/time.h>
 #  if defined(Plan9) || defined(M_UNIX) || defined(_IBMR2) || \
       defined(_SEQUENT_) || defined(__GNUC__) || defined(__INTEL_COMPILER) ||\
@@ -76,7 +78,7 @@ struct timezone {
 #endif
 
 /* Some System V environments, and Posix environments, need <sys/times.h>. */
-#ifdef HAVE_SYS_TIMES_H
+#if defined(HAVE_SYS_TIMES_H) && HAVE_SYS_TIMES_H == 1
 #  include <sys/times.h>
 #  define use_times_for_usertime 1
                 /* Posix 1003.1b-1993 section 4.8.1.5 says that
