@@ -1,22 +1,24 @@
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
+   All Rights Reserved.
+
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+
 /*
     jbig2dec
-    
-    Copyright (C) 2003 Artifex Software, Inc.
-    
-    This software is distributed under license and may not
-    be copied, modified or distributed except as expressly
-    authorized under the terms of the license contained in
-    the file LICENSE in this distribution.
-                                                                                
-    For information on commercial licensing, go to
-    http://www.artifex.com/licensing/ or contact
-    Artifex Software, Inc.,  101 Lucas Valley Road #110,
-    San Rafael, CA  94903, U.S.A., +1(415)492-9861.
-
-    $Id: jbig2_metadata.h,v 1.4 2007/09/11 15:24:52 Arabidopsis Exp $
 */
 
 
+
 #ifndef _JBIG2_METADATA_H
 #define _JBIG2_METADATA_H
 
@@ -43,9 +45,9 @@ struct _Jbig2Metadata {
 };
 
 /* these bits can go to jbig2_priv.h */
-int jbig2_parse_comment_ascii(Jbig2Ctx *ctx, Jbig2Segment *segment,
+int jbig2_comment_ascii(Jbig2Ctx *ctx, Jbig2Segment *segment,
                                 const uint8_t *segment_data);
-int jbig2_parse_comment_unicode(Jbig2Ctx *ctx, Jbig2Segment *segment,
+int jbig2_comment_unicode(Jbig2Ctx *ctx, Jbig2Segment *segment,
                                const uint8_t *segment_data);
 
 #endif /* _JBIG2_METADATA_H */
