@@ -1,18 +1,20 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
+
 /* dwnodll.c */
-/* $Id$ */
 
 #define STRICT
 #include <windows.h>
@@ -35,6 +37,7 @@ int load_dll(GSDLL *gsdll, char *last_error, int len)
     gsdll->set_poll = &gsapi_set_poll;
     gsdll->set_display_callback = &gsapi_set_display_callback;
     gsdll->init_with_args = &gsapi_init_with_args;
+    gsdll->set_arg_encoding = &gsapi_set_arg_encoding;
     gsdll->run_string = &gsapi_run_string;
     gsdll->exit = &gsapi_exit;
     gsdll->set_visual_tracer = &gsapi_set_visual_tracer;
