@@ -70,13 +70,15 @@ struct gs_main_instance_s {
     long base_time[2];		/* starting usertime */
     void *readline_data;	/* data for gp_readline */
     ref error_object;		/* Use by gsapi_*() */
-#if 1
-    /* needs to be removed */
+    int (*get_codepoint)(FILE *file, const char **astr);
+                                /* Get next 'unicode' codepoint (in
+                                 * GS_NO_UTF8 builds, these will actually
+                                 * just be 8 bit clean values). */
     display_callback *display;	/* callback structure for display device */
-#endif
-
     /* The following are updated dynamically. */
     i_ctx_t *i_ctx_p;		/* current interpreter context state */
+    char *saved_pages_initial_arg;	/* used to defer processing of --saved-pages=begin... */
+    bool saved_pages_test_mode;	/* for regression testing of saved-pages */
 };
 
 /*
@@ -84,7 +86,9 @@ struct gs_main_instance_s {
  * must include gconfig.h, because of SEARCH_HERE_FIRST.
  */
 #define gs_main_instance_default_init_values\
-  0, 20000, 0, 0, -1, 0, SEARCH_HERE_FIRST, 1
+  0/* heap */, 20000/* chunk_size */, 0/* name_table_size */, 0/* run_buffer_size */,\
+  -1/* init_done */, 0/* user_errors */, SEARCH_HERE_FIRST/* duh */, 1/* run_start */,
+
 extern const gs_main_instance gs_main_instance_init_values;
 
 #endif /* iminst_INCLUDED */
