@@ -62,7 +62,9 @@ zfont_global_glyph_code(const gs_memory_t *mem, gs_const_string *gstr, gs_glyph
 static int
 zfont_init(i_ctx_t *i_ctx_p)
 {
-    ifont_dir = gs_font_dir_alloc2(imemory, imemory->non_gc_memory);
+    ifont_dir = gs_font_dir_alloc2(imemory->stable_memory, imemory->non_gc_memory);
+    if (ifont_dir == NULL)
+        return gs_error_VMerror;
     ifont_dir->ccache.mark_glyph = zfont_mark_glyph_name;
     ifont_dir->global_glyph_code = zfont_global_glyph_code;
     return gs_register_struct_root(imemory, NULL, (void **)&ifont_dir,
